; ( VGA Console )

; ( KEY, a generic keyboard driver )
key:	  dup				; key
	  xor eax,eax		;  clear eax
.1      in al,64h       	;  Is any data waiting?
	  test al,1			;  Is character = ASCII 0?
	  jz .1			;  Yes? Try again
        in al,60h       	;  Otherwise, read scancode
        xor edx,edx		;  edx: 0=make, 1=break
        test al,80h		;  Is character = HEX 80h?
        jz .2			;  Skip the next line
        inc edx			;  Update edx
.2      and al,7Fh		;  Filters to handle
        cmp al,39h      	;  the ignored keys
	  ja .1			;  We just try another key
        mov ecx,[board]		;  Load the keymap
        mov al,[ecx+eax]	;  Get the key ASCII char
	  or al,al			;  Is is = 0?
        js .shift			;  No, use CAPITALS
        jz .1  			;  Ignore 0's
        or dl,dl			;  Filter for break code
        jnz .1 			;  Ignore break code
 	  ret
.shift  mov ecx,[edx*4 + .shifts]	;  Load the CAPITAL keymap
        mov [board],ecx			;  Store into BOARD pointer
        jmp short .1			;  And try again
.shifts dd shift,alpha
variable board, alpha
alpha:
  db 0,27,"1234567890-=",8              ;00-0E
  db 9,"qwertyuiop[]",10                ;0F-1C
  db 0,"asdfghjkl;'`"                   ;1D-29
  db -1,"\zxcvbnm,./",-1,"*",0,32,-2    ;2A-3A
shift:
  db 0,27,"!@#$%^&*()_+",8              ;00-0E
  db 9,"QWERTYUIOP{}",10                ;0F-1C
  db 0,'ASDFGHJKL:"~'                   ;1D-29
  db -1,"|ZXCVBNM<>?",-1,"*",0,32,-2    ;2A-3A
; ( EMIT, basic character output )
emit:
	push esi			; emit
	push edi			;  Store a few registers
	push ecx
	push edx
	call ?cr			;  Wrap to a new line?
	and eax,0xff			;  Filter byte
	imul eax,ih*iw/8		;  byte/char in FONT
	lea esi,[font+eax]		;  Load byte from FONT
	call clip			;  Clip the drawing region
	mov edx,[fore] 			;  color (2 = green)
	mov ecx,ih		;  Height of character
.line	push ecx			;  Store height
	lodsb			 	;  al=font line
	mov ecx,iw			;  Width of character
.pix	shl al,1			;  Begin drawing the character
	jnc .1
	mov [edi],dl			;  Store a pixel
.1	inc edi
	loop .pix			;  Is this line done?
	add edi,hp-iw			;  Yes? Then do next line
	pop ecx				;  Restore height/width
	loop .line			;  Are we done?
	pop edx				;  Yes!
	pop ecx				;  Restore the registers
	pop edi
	pop esi
	drop				; drop the character from stack
      add [xy],dword (iw<<16)		; And advance one space
ret
box:
	push ecx
	push edx
	push edi
	call clip
	upop ecx		  ;h
	mov edx,eax		  ;w
      mov eax,dword [fore]           ;color
.1	push ecx
        mov ecx,edx		  ;w
	rep stosb
	add edi,hp
	sub edi,edx
	pop ecx
	loop .1
	pop edi
	pop edx
	pop ecx
	drop
	ret

clip:	mov edi,[xy]
	mov ecx,edi
	test cx,cx
	jns .1
	xor ecx,ecx
.1	and ecx,0xffff
	imul ecx,hp		;320*2 for 16-bit gfx
	sar edi,16
	jns .2
	xor edi,edi
.2	lea edi,[edi+ecx]	;edi*2 for 16-bit
        add edi,dword 0xA0000
	ret

; ( Interface IV - That blue bar at the bottom of the screen )
interface:
      push dword [xy]
	push dword [fore]
	$color 7Fh		;erase bottom line
      $at 0,(vc-1)*ih
	upsh hp
      upsh ih
	call box
	$color 14		;text color
      $at 0,(vc-1)*ih
    	upsh tib
	upsh [tp]
	sub eax,[esi]
	dup		;set x coordinate to right-justify text
      mov eax,hc-1
	sub eax,[esi]
      imul eax,iw
	mov [xy+2],ax
      drop
	call type
	pop dword [fore]
	pop dword [xy]
.3	ret

font: incbin 'drivers/font'
